require 'minitest/test'
require_relative '../lib/app_copyr'
require_relative 'test_classes'
require "minitest/autorun"

describe AppCopyr do
  def setup
    @bad_subject = EClass.new
    @good_subject = GClass.new
    @already_exists = EExistsClass.new

    unless Dir.exists?($test_dir)
      puts "Creating source test app"
      system("rails new #{$test_dir}")
    end
  end

  it "test_raise_error_when_source_missing" do
    assert_raises AppCopyr::DirMissing do
      @bad_subject.main
    end
  end

  it "raises an error when the destination exists" do
    assert_raises AppCopyr::DistinationExists do
      @already_exists.main
    end
  end

  it 'has the correct new app name' do
    assert_match /SomeNotRealDest/, @bad_subject.new_app_name
  end

  it 'run_copy runs without error' do
    @good_subject.run_copy
  end

  it 'should run through completely without error' do
    `rails new #{$final_test_dir}`
    # Capture output to stdout, otherwise it goes to stderr, works on a mac...
    output = `bin/app-copyr copy --source #{$final_test_dir} --dest #{$final_test_out_dir} > /dev/stdout 2>&1`
    refute_match(/No value provided for required options/, output)
    assert_equal($?.exitstatus, 0)
  end

  it 'creates a functional rails app when done' do
    `rails new #{$final_test_dir}`
    `bin/app-copyr copy --source=#{$final_test_dir} --dest=#{$final_test_out_dir} > /dev/stdout 2>&1`
    Dir.chdir($final_test_out_dir){
      `rake -T`
      assert_equal($?.exitstatus, 0)
    }
  end

  def teardown
    `rm -rf #{$good_dest_dir}`
    `rm -rf #{$already_exists_dir}`
    `rm -rf #{$final_test_dir}`
    `rm -rf #{$final_test_out_dir}`
  end

end